/* DynRelateDS.p  
    Last Updated- DL 5/04/04 
    
    Overview of code:
    The purpose of this demonstation code is to show one way of using 4GL dynamic coding 
    using the OpenEdge .NET Open Client.  The code can be run
    on the AppServer session-free and accessed by a .NET Client client using the session
    free model.
    
    Input parameters include the table sources, the keys for the sources, the relationships
    between the sources.  Examples of these values for the Sports2000 database include:
        For Customer and Order tables:
    			pcSources	 = "Customer,Order"
				pcKeys = "CustNum,OrderNum"
				pcRelations  = "CustNum,CustNum"
        For Order and Orderline tables:
        		pcSources	 = "Order,OrderLine";
				pcKeys = "OrderNum,LineNum";
				pcRelations  = "OrderNum,OrderNum";
                
    Also as input parameters a range is specificed for the first table source.  The range is 
    specified as greater than or equal to and less than or equal to using piGE and 
    piLE respectively.  
    
    The output parameter is the ProDataSet that was created and includes the two input tables 
    with the relation defined between them.  The tables are populated based on the input range.
    
    The 'OUTPUT TO', 'OUTPUT CLOSE', and DISPLAY statments are optional.  They are included 
    in the example since they can be useful in quickly identifying some problems if the code 
    isn't working as expected.  The output is created in C:\OpenEdge\wrk.  This path
    can be changed to a directory that exists on the system where the code is run.
*/

DEFINE INPUT  PARAMETER pcSources   AS CHARACTER  NO-UNDO.
DEFINE INPUT  PARAMETER pcKeys      AS CHARACTER  NO-UNDO.
DEFINE INPUT  PARAMETER pcRelations AS CHARACTER  NO-UNDO.
DEFINE INPUT  PARAMETER piGE        AS INTEGER    NO-UNDO.
DEFINE INPUT  PARAMETER piLE        AS INTEGER    NO-UNDO.
DEFINE OUTPUT PARAMETER DATASET-HANDLE phDataSet.

DEFINE VARIABLE iEntry      AS INTEGER    NO-UNDO.
DEFINE VARIABLE hDataSource AS HANDLE     NO-UNDO.
DEFINE VARIABLE hBuffer     AS HANDLE     NO-UNDO.
DEFINE VARIABLE hQuery      AS HANDLE     NO-UNDO.
DEFINE VARIABLE hTable      AS HANDLE     NO-UNDO.

OUTPUT TO "C:\OpenEdge\WRK\DynRelateDSOUTT.txt".  
DISPLAY "DynRelateDS.p" SKIP.

CREATE DATASET phDataSet.
 
tableLoop: 
DO iEntry = 1 TO NUM-ENTRIES(pcSources):
    CREATE TEMP-TABLE hTable.
    hTable:CREATE-LIKE(ENTRY(iEntry, pcSources)).
    hTable:TEMP-TABLE-PREPARE("Sport" + ENTRY(iEntry, pcSources)).
    phDataSet:ADD-BUFFER(hTable:DEFAULT-BUFFER-HANDLE).
END.
 
DO iEntry = 1 TO NUM-ENTRIES(pcSources):
    CREATE DATA-SOURCE hDataSource.
    CREATE BUFFER hBuffer FOR TABLE ENTRY(iEntry, pcSources).
    hDataSource:ADD-SOURCE-BUFFER(hBuffer, ENTRY(iEntry,pcKeys)).
    phDataSet:GET-BUFFER-HANDLE(iEntry):ATTACH-DATA-SOURCE(hDataSource).
     
    IF iEntry = 1 THEN
    DO:
        CREATE QUERY hQuery.
        hQuery:ADD-BUFFER(hBuffer).
        hQuery:QUERY-PREPARE("FOR EACH " + ENTRY(1, pcSources) +
                             " NO-LOCK WHERE " + ENTRY(1, pcKeys) + " GE " + STRING(piGE) + 
                             " AND " + ENTRY(1, pcKeys) + " LE " + STRING(piLE)).
        hDataSource:QUERY = hQuery.
    END.
END.

phDataSet:ADD-RELATION(phDataSet:GET-BUFFER-HANDLE(1), 
                       phDataSet:GET-BUFFER-HANDLE(2),
                       pcRelations).

phDataSet:FILL().
OUTPUT CLOSE.   
RETURN. 
